"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@babel/core");
const getTypeName_1 = __importDefault(require("./getTypeName"));
const NATIVE_BUILT_INS = ['Date', 'Error', 'RegExp', 'Map', 'WeakMap', 'Set', 'WeakSet', 'Promise'];
function hasCustomPropTypeSuffix(name, suffixes) {
    return !!suffixes && suffixes.some(suffix => name.endsWith(suffix));
}
function isReactTypeMatch(name, type, reactImportedName) {
    return name === type || name === `React.${type}` || name === `${reactImportedName}.${type}`;
}
function wrapIsRequired(propType, optional) {
    return optional ? propType : core_1.types.memberExpression(propType, core_1.types.identifier('isRequired'));
}
function createMember(value, propTypesImportedName) {
    return core_1.types.memberExpression(core_1.types.identifier(propTypesImportedName), value);
}
function createCall(value, args, propTypesImportedName) {
    return core_1.types.callExpression(createMember(value, propTypesImportedName), args);
}
function convert(type, state) {
    const { reactImportedName, propTypes } = state;
    const propTypesImportedName = propTypes.defaultImport;
    // Remove wrapping parens
    if (core_1.types.isTSParenthesizedType(type)) {
        type = type.typeAnnotation;
    }
    state.propTypes.count += 1;
    // any -> PropTypes.any
    if (core_1.types.isTSAnyKeyword(type)) {
        return createMember(core_1.types.identifier('any'), propTypesImportedName);
        // string -> PropTypes.string
    }
    else if (core_1.types.isTSStringKeyword(type)) {
        return createMember(core_1.types.identifier('string'), propTypesImportedName);
        // number -> PropTypes.number
    }
    else if (core_1.types.isTSNumberKeyword(type)) {
        return createMember(core_1.types.identifier('number'), propTypesImportedName);
        // boolean -> PropTypes.bool
    }
    else if (core_1.types.isTSBooleanKeyword(type)) {
        return createMember(core_1.types.identifier('bool'), propTypesImportedName);
        // symbol -> PropTypes.symbol
    }
    else if (core_1.types.isTSSymbolKeyword(type)) {
        return createMember(core_1.types.identifier('symbol'), propTypesImportedName);
        // object -> PropTypes.object
    }
    else if (core_1.types.isTSObjectKeyword(type)) {
        return createMember(core_1.types.identifier('object'), propTypesImportedName);
        // (() => void) -> PropTypes.func
    }
    else if (core_1.types.isTSFunctionType(type)) {
        return createMember(core_1.types.identifier('func'), propTypesImportedName);
        // React.ReactNode -> PropTypes.node
        // React.ReactElement -> PropTypes.element
        // React.MouseEvent -> PropTypes.object
        // React.MouseEventHandler -> PropTypes.func
        // React.Ref -> PropTypes.oneOfType()
        // JSX.Element -> PropTypes.element
        // FooShape, FooPropType -> FooShape, FooPropType
        // Date, Error, RegExp -> Date, Error, RegExp
        // CustomType -> PropTypes.any
    }
    else if (core_1.types.isTSTypeReference(type)) {
        const name = getTypeName_1.default(type.typeName);
        // node
        if (isReactTypeMatch(name, 'ReactText', reactImportedName) ||
            isReactTypeMatch(name, 'ReactNode', reactImportedName) ||
            isReactTypeMatch(name, 'ReactType', reactImportedName)) {
            return createMember(core_1.types.identifier('node'), propTypesImportedName);
            // function
        }
        else if (isReactTypeMatch(name, 'ComponentType', reactImportedName) ||
            isReactTypeMatch(name, 'ComponentClass', reactImportedName) ||
            isReactTypeMatch(name, 'StatelessComponent', reactImportedName)) {
            return createMember(core_1.types.identifier('func'), propTypesImportedName);
            // element
        }
        else if (isReactTypeMatch(name, 'Element', 'JSX') ||
            isReactTypeMatch(name, 'ReactElement', reactImportedName) ||
            isReactTypeMatch(name, 'SFCElement', reactImportedName)) {
            return createMember(core_1.types.identifier('element'), propTypesImportedName);
            // oneOfType
        }
        else if (isReactTypeMatch(name, 'Ref', reactImportedName)) {
            return createCall(core_1.types.identifier('oneOfType'), [
                core_1.types.arrayExpression([
                    createMember(core_1.types.identifier('string'), propTypesImportedName),
                    createMember(core_1.types.identifier('func'), propTypesImportedName),
                    createMember(core_1.types.identifier('object'), propTypesImportedName),
                ]),
            ], propTypesImportedName);
            // func
        }
        else if (name.endsWith('Handler')) {
            return createMember(core_1.types.identifier('func'), propTypesImportedName);
            // object
        }
        else if (name.endsWith('Event')) {
            return createMember(core_1.types.identifier('object'), propTypesImportedName);
            // native built-ins
        }
        else if (NATIVE_BUILT_INS.includes(name)) {
            return createCall(core_1.types.identifier('instanceOf'), [core_1.types.identifier(name)], propTypesImportedName);
            // custom prop type variables (must be last)
        }
        else if (hasCustomPropTypeSuffix(name, state.options.customPropTypeSuffixes)) {
            return core_1.types.identifier(name);
        }
        // any (we need to support all these in case of unions)
        return createMember(core_1.types.identifier('any'), propTypesImportedName);
        // [] -> PropTypes.arrayOf(), PropTypes.array
    }
    else if (core_1.types.isTSArrayType(type)) {
        const args = convertArray([type.elementType], state);
        return args.length > 0
            ? createCall(core_1.types.identifier('arrayOf'), args, propTypesImportedName)
            : createMember(core_1.types.identifier('array'), propTypesImportedName);
        // {} -> PropTypes.object
        // { [key: string]: string } -> PropTypes.objectOf(PropTypes.string)
        // { foo: string } -> PropTypes.shape({ foo: PropTypes.string })
    }
    else if (core_1.types.isTSTypeLiteral(type)) {
        // object
        if (type.members.length === 0) {
            return createMember(core_1.types.identifier('object'), propTypesImportedName);
            // objectOf
        }
        else if (type.members.length === 1 && core_1.types.isTSIndexSignature(type.members[0])) {
            const index = type.members[0];
            if (index.typeAnnotation && index.typeAnnotation.typeAnnotation) {
                const result = convert(index.typeAnnotation.typeAnnotation, state);
                if (result) {
                    return createCall(core_1.types.identifier('objectOf'), [result], propTypesImportedName);
                }
            }
            // shape
        }
        else {
            return createCall(core_1.types.identifier('shape'), [
                core_1.types.objectExpression(convertListToProps(type.members.filter(member => core_1.types.isTSPropertySignature(member)), state)),
            ], propTypesImportedName);
        }
        // string | number -> PropTypes.oneOfType([PropTypes.string, PropTypes.number])
        // 'foo' | 'bar' -> PropTypes.oneOf(['foo', 'bar'])
    }
    else if (core_1.types.isTSUnionType(type) || core_1.types.isTSIntersectionType(type)) {
        const isAllLiterals = type.types.every(param => core_1.types.isTSLiteralType(param));
        let label;
        let args;
        if (isAllLiterals) {
            args = type.types.map(param => param.literal);
            label = core_1.types.identifier('oneOf');
        }
        else {
            args = convertArray(type.types, state);
            label = core_1.types.identifier('oneOfType');
        }
        if (label && args.length > 0) {
            return createCall(label, [core_1.types.arrayExpression(args)], propTypesImportedName);
        }
    }
    state.propTypes.count -= 1;
    return null;
}
function convertArray(types, state) {
    const propTypes = [];
    types.forEach(type => {
        const prop = convert(type, state);
        if (prop) {
            propTypes.push(prop);
        }
    });
    return propTypes;
}
function convertListToProps(properties, state, defaultProps = []) {
    const propTypes = [];
    properties.forEach(property => {
        if (!property.typeAnnotation) {
            return;
        }
        const propType = convert(property.typeAnnotation.typeAnnotation, state);
        if (propType) {
            propTypes.push(core_1.types.objectProperty(property.key, wrapIsRequired(propType, property.optional || defaultProps.includes(property.key.name))));
        }
    });
    return propTypes;
}
function convertToPropTypes(types, typeNames, state, defaultProps) {
    const properties = [];
    typeNames.forEach(typeName => {
        if (types[typeName]) {
            properties.push(...convertListToProps(types[typeName], state, defaultProps));
        }
    });
    return properties;
}
exports.default = convertToPropTypes;
