"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const helper_plugin_utils_1 = require("@babel/helper-plugin-utils");
const helper_module_imports_1 = require("@babel/helper-module-imports");
const plugin_syntax_typescript_1 = __importDefault(require("@babel/plugin-syntax-typescript"));
const core_1 = require("@babel/core");
// import ts from 'typescript';
const addToClass_1 = __importDefault(require("./addToClass"));
const addToFunctionOrVar_1 = __importDefault(require("./addToFunctionOrVar"));
const extractTypeProperties_1 = __importDefault(require("./extractTypeProperties"));
const upsertImport_1 = __importDefault(require("./upsertImport"));
const BABEL_VERSION = 7;
function isNotTS(name) {
    return name.endsWith('.js') || name.endsWith('.jsx');
}
exports.default = helper_plugin_utils_1.declare((api, options) => {
    api.assertVersion(BABEL_VERSION);
    return {
        inherits: plugin_syntax_typescript_1.default,
        manipulateOptions(opts, parserOptions) {
            // Inheriting the syntax doesn't seem to define these
            parserOptions.plugins.push('jsx');
        },
        post() {
            // Free up any memory we're hogging
            this.state = null;
        },
        pre() {
            // Setup initial state
            this.state = {
                airbnbPropTypes: {
                    count: 0,
                    forbidImport: '',
                    hasImport: false,
                    namedImports: [],
                },
                componentTypes: {},
                filePath: '',
                options,
                propTypes: {
                    count: 0,
                    defaultImport: '',
                    hasImport: false,
                },
                reactImportedName: '',
            };
        },
        visitor: {
            Program: {
                enter(programPath, { filename }) {
                    const state = this.state;
                    state.filePath = filename;
                    if (isNotTS(filename)) {
                        return;
                    }
                    // Find existing `react` and `prop-types` imports
                    programPath.node.body.forEach(node => {
                        if (!core_1.types.isImportDeclaration(node)) {
                            return;
                        }
                        if (node.source.value === 'prop-types') {
                            const response = upsertImport_1.default(node, {
                                checkForDefault: 'PropTypes',
                            });
                            state.propTypes.hasImport = true;
                            state.propTypes.defaultImport = response.defaultImport;
                        }
                        if (node.source.value === 'airbnb-prop-types') {
                            const response = upsertImport_1.default(node, { checkForNamed: 'forbidExtraProps' });
                            state.airbnbPropTypes.hasImport = true;
                            state.airbnbPropTypes.namedImports = response.namedImports;
                            state.airbnbPropTypes.forbidImport = response.namedImport;
                        }
                        if (node.source.value === 'react') {
                            const response = upsertImport_1.default(node, {
                                checkForDefault: 'React',
                            });
                            state.reactImportedName = response.defaultImport;
                        }
                    });
                    // Add `prop-types` import if it does not exist.
                    // We need to do this without a visitor as we need to modify
                    // the AST before anything else has can run.
                    if (!state.propTypes.hasImport && state.reactImportedName) {
                        state.propTypes.defaultImport = helper_module_imports_1.addDefault(programPath, 'prop-types', {
                            nameHint: 'pt',
                        }).name;
                    }
                    if (!state.airbnbPropTypes.hasImport &&
                        state.reactImportedName &&
                        options.forbidExtraProps) {
                        state.airbnbPropTypes.forbidImport = helper_module_imports_1.addNamed(programPath, 'forbidExtraProps', 'airbnb-prop-types').name;
                        state.airbnbPropTypes.count += 1;
                    }
                    // Abort early if we're definitely not in a file that needs conversion
                    if (!state.propTypes.defaultImport && !state.reactImportedName) {
                        return;
                    }
                    programPath.traverse({
                        // airbnbPropTypes.componentWithName()
                        CallExpression({ node }) {
                            const { namedImports } = state.airbnbPropTypes;
                            if (options.forbidExtraProps &&
                                core_1.types.isIdentifier(node.callee) &&
                                namedImports.includes(node.callee.name)) {
                                state.airbnbPropTypes.count += 1;
                            }
                        },
                        // `class Foo extends React.Component<Props> {}`
                        // @ts-ignore
                        'ClassDeclaration|ClassExpression': (path) => {
                            const { node } = path;
                            // prettier-ignore
                            const valid = node.superTypeParameters && (
                            // React.Component, React.PureComponent
                            (core_1.types.isMemberExpression(node.superClass) &&
                                core_1.types.isIdentifier(node.superClass.object, { name: state.reactImportedName }) && (core_1.types.isIdentifier(node.superClass.property, { name: 'Component' }) ||
                                core_1.types.isIdentifier(node.superClass.property, { name: 'PureComponent' }))) ||
                                // Component, PureComponent
                                (state.reactImportedName && (core_1.types.isIdentifier(node.superClass, { name: 'Component' }) ||
                                    core_1.types.isIdentifier(node.superClass, { name: 'PureComponent' }))));
                            if (valid) {
                                addToClass_1.default(node, state);
                            }
                        },
                        // `function Foo(props: Props) {}`
                        FunctionDeclaration(path) {
                            const { node } = path;
                            const param = node.params[0];
                            // prettier-ignore
                            const valid = param &&
                                state.reactImportedName &&
                                node.id.name.match(/^[A-Z]/u) && (
                            // (props: Props)
                            (core_1.types.isIdentifier(param) && param.typeAnnotation) ||
                                // ({ ...props }: Props)
                                (core_1.types.isObjectPattern(param) && param.typeAnnotation));
                            if (valid) {
                                addToFunctionOrVar_1.default(path, node.id.name, state);
                            }
                        },
                        // airbnbPropTypes.nonNegativeInteger
                        Identifier({ node }) {
                            const { namedImports } = state.airbnbPropTypes;
                            if (options.forbidExtraProps && namedImports.includes(node.name)) {
                                state.airbnbPropTypes.count += 1;
                            }
                        },
                        // PropTypes.*
                        MemberExpression({ node }) {
                            if (core_1.types.isIdentifier(node.object, { name: state.propTypes.defaultImport })) {
                                state.propTypes.count += 1;
                            }
                        },
                        // `interface FooProps {}`
                        TSInterfaceDeclaration({ node }) {
                            state.componentTypes[node.id.name] = extractTypeProperties_1.default(node, state.componentTypes);
                        },
                        // `type FooProps = {}`
                        TSTypeAliasDeclaration({ node }) {
                            state.componentTypes[node.id.name] = extractTypeProperties_1.default(node, state.componentTypes);
                        },
                        // `const Foo: React.SFC<Props> = () => {};`
                        VariableDeclaration(path) {
                            const { node } = path;
                            if (node.declarations.length === 0) {
                                return;
                            }
                            const id = node.declarations[0].id;
                            if (!id.typeAnnotation || !id.typeAnnotation.typeAnnotation) {
                                return;
                            }
                            const type = id.typeAnnotation.typeAnnotation;
                            // prettier-ignore
                            const valid = core_1.types.isTSTypeReference(type) &&
                                type.typeParameters &&
                                type.typeParameters.params.length > 0 && (
                            // React.SFC, React.StatelessComponent
                            (core_1.types.isTSQualifiedName(type.typeName) &&
                                core_1.types.isIdentifier(type.typeName.left, { name: state.reactImportedName }) &&
                                (core_1.types.isIdentifier(type.typeName.right, { name: 'SFC' }) ||
                                    core_1.types.isIdentifier(type.typeName.right, { name: 'StatelessComponent' }))) ||
                                // SFC, StatelessComponent
                                (state.reactImportedName && (core_1.types.isIdentifier(type.typeName, { name: 'SFC' }) ||
                                    core_1.types.isIdentifier(type.typeName, { name: 'StatelessComponent' }))));
                            if (valid) {
                                addToFunctionOrVar_1.default(path, id.name, state);
                            }
                        },
                    });
                },
                exit(path, { filename }) {
                    const state = this.state;
                    if (isNotTS(filename)) {
                        return;
                    }
                    // Remove the `prop-types` import of no components exist,
                    // and be sure not to remove pre-existing imports.
                    path.get('body').forEach(bodyPath => {
                        if (state.propTypes.count === 0 &&
                            core_1.types.isImportDeclaration(bodyPath.node) &&
                            bodyPath.node.source.value === 'prop-types') {
                            bodyPath.remove();
                        }
                    });
                },
            },
        },
    };
});
